from datetime import datetime,timedelta

from django.utils import simplejson
from django.conf import settings

from georemindme.models import User,GeoUser, POI, Point, Business, Alert

from rpc_server.models import UserRPC
from rpc_server.decorators import jsonrpc_function, jsonrpc_logged_function
from rpc_server.exceptions import GeoException



@jsonrpc_function
def login(email, password):
	"""
		Log a user and creates a new UserRPC session ID
		The user_id should be send with all the request.
	"""
	u = User.objects.get_by_email(email=email)
	if u is not None:
		if u.check_password(password):
			
			if not u.is_confirmed() and u.created+timedelta(days=settings.NO_CONFIRM_ALLOW_DAYS) < datetime.now():
				u.send_confirm_code()
				raise Exception( GeoException.NO_CONFIRMED )
				
			urpc = UserRPC.objects.get_by_email(email=u.email)
			if urpc is None:
				urpc = UserRPC(email=email,realuser = u)
			urpc.put()			
			return urpc.session_id
	raise Exception( GeoException.BAD_EMAIL_PASSWORD )

@jsonrpc_function
def register(email, password):
	"""
		Register a user
		returns True if sucessful
	"""
	try:
		u = GeoUser(email=email, password=password)
		u.send_confirm_code()
		u.put()
		if u is not None:
			return True	
	except:
		pass
	raise Exception( GeoException.BAD_EMAIL_PASSWORD )

@jsonrpc_function
def sync(session_id, last_sync, modified):
	"""
		Sync with devices
	"""	
	u = UserRPC.objects.get_by_id(session_id)
	if u is None or not u.is_valid():
		raise Exception( GeoException.INVALID_SESSION )

	# 1) parse last_sync and modified to datetime
	
	
	for a in modified:
		if a.get('point'):
			poi = Point( **(a['point']) )
			del a['point']
		elif a.get('business'):
			poi = Business( **(a['business']) )
			del a['business']
		else:
			raise Exception( GeoException.INVALID_REQUEST )
		
		alert = Alert(poi=poi,**a) # fill the alert data

		if not alert.modified: # the alert is new
			alert.put()
		else:
			old = Alert.objects.get_by_id_user(alert.id, u.realuser)
			
			if not old:
				raise Exception( GeoException.INVALID_REQUEST )
			
			poi.put()
			
			old.name = alert.name
			old.starts = alert.starts
			old.ends = alert.ends
			old.description = alert.description
			old.poi = poi
			old.created = alert.created
			old.modified = alert.modified
			old.done_when = alert.done_when
			if old.is_done() != alert.done:
				old.toggle_done()
			if old.is_active() != alert.active:
				old.toggle_active() 
			old.set_distance(alert.distance)
			old.put()
			# return the alerts modified after last sync
	def compare(a, b):
		return cmp(a.modified, b.modified) # compare as integers
	
	alerts = Alert.objects.get_by_last_sync(u.realuser, last_sync)
	alerts.sort(compare)
	ret = []
	for a in alerts:
		ret.append(dict(id=str(alert.key(),
					points=[ (alert.poi.point.lat,alert.poi.point.lon) ],
					name = alert.name,
					starts = alert.starts,
					ends = alert.ends,
					created = alert.created,
					modified = alert.modified,
					done_when = alert.done_when,
					done = alert.is_done(),
					active = alert.is_active(),
					))
		)
	return (datetime.datetime.now(), ret)



@jsonrpc_function
def getProximityAlerts(session_id, lat, long):
	"""
		Get the Alerts near to the user position
	"""
	
	u = UserRPC.objects.get_by_id(session_id)
	if u is None or not u.is_valid():
		raise Exception( GeoException.INVALID_SESSION )
		
	from geobox import proximity_alerts

	alerts = proximity_alerts(u.realuser, lat, long)
	##return simplejson.dumps( [ a.__dict__ for a in alerts ] )
	
	# now only works for Point alerts (no Business alerts)
	ret = [
		dict(id=str(alert.key()), # alert's key
		points=[ (alert.poi.point.lat,alert.poi.point.lon) ] # point of the alert
		
		) for alert in alerts]
	
	return ret
	
	


